@@ -32,6 +32,12 @@ setGeneric("getY",
 setGeneric("getValues",
     function(object, ...){standardGeneric("getValues")})
 
+#' @name generics-accessors
+#' @aliases getCoherency
+#' @export
+setGeneric("getCoherency",
+    function(object, ...){standardGeneric("getCoherency")})
+
 #' @name generics-accessors
 #' @aliases getIsRankBased
 #' @export
@@ -123,6 +129,11 @@ setGeneric("getTs", function(object, ...){standardGeneric("getTs")})
 
 ## Class-SmoothedPG
 
+#' @name generics-accessors
+#' @aliases getCoherencySdNaive
+#' @export
+setGeneric("getCoherencySdNaive", function(object, ...){standardGeneric("getCoherencySdNaive")})
+
 #' @name generics-accessors
 #' @aliases getSdNaive
 #' @export
